from asw_sdk.ConfigurationManager import ConfigurationManager
from asw_sdk.WebClient import WebClient
from enum import Enum

import os
import json
from datetime import datetime, timezone


class LogLevel(Enum):
    INFO = 1900
    FAILED = 1800
    SUCCESS = 2000


def generate_json_parseable_message(message: str):
    message = message.replace('\\', '\\\\')
    message = message.replace('\b', '\\b')
    message = message.replace('\f', '\\f')
    message = message.replace('\n', '\\n')
    message = message.replace('\r', '\\r')
    message = message.replace('\t', '\\t')
    message = message.replace('"', '\\"')
    return message


class Utils:
    def __init__(self, override_config=None, overriden_config_node=None):
        self._config_manager = None
        if overriden_config_node and override_config:
            self._config_manager = ConfigurationManager(overriden_config_node, override_config)
        else:
            self._config_manager = ConfigurationManager(os.environ['PYTHON_CONFIG_NODE'])
        self._web_client = WebClient("exec", self._config_manager)
        self._web_client.refresh_token()

    def print(self, message, level=LogLevel.INFO):
        operations_key = self._config_manager.get_operations_key()
        now = datetime.now(timezone.utc)
        filter_id = self._config_manager.get_filter_id() or "null"
        job_id = self._config_manager.get_currently_executing_job_id() or "null"
        message = generate_json_parseable_message(message)
        filter_log_request_body_as_string = f'{{"displayMessage": null, "filterId": {filter_id}, "filterName": "{self._config_manager.get_currently_executing_step_name()}", "filterProgressStatusValue": {level.value}, "id": null, "jobId": {job_id}, "logDate": "{now.isoformat()}", "messageError": "{message}", "operationsKey": "{operations_key}"}}'
        print('filter_log_request_body_as_string', filter_log_request_body_as_string)
        filter_progress_log_endpoint = f'/job-stream/external-filter-progress'
        filter_log_request_body_as_json = json.loads(filter_log_request_body_as_string)
        print('filter_log_request_body_as_json', filter_log_request_body_as_json)
        self._web_client.post_entity(filter_progress_log_endpoint, json_data=filter_log_request_body_as_json, headers={})


if __name__ == '__main__':
    os.environ['PYTHON_CONFIG'] = "{    \\\"TOKEN\\\" : \\\"eyJob3N0TmFtZSI6Imh0dHBzOi8vYWxwaGEzLXRlbmFudDMtcnVjaGl0LmFscGhhLmxvY2FsOjk5OTgiLCJyZWZyZXNoVG9rZW4iOiJyX2xvdWpETmxYWEFwRjdXelAiLCJ3c0hvc3ROYW1lIjoid3NzOi8vYWxwaGEzLXRlbmFudDMtcnVjaGl0LndzLmFscGhhLmxvY2FsOjk5OTgvd3MifQ==\\\",    \\\"nodes\\\" : {        \\\"b179a8e34f9f4cb4a319bad9b4e85a5e\\\" : {            \\\"applicationDataTypeId\\\" : 4104,            \\\"applicationGroupId\\\" : 106,            \\\"cittaAgent\\\" : {                \\\"launcher\\\" : {                    \\\"pipelineConf\\\" : {                        \\\"filterName\\\" : \\\"smm-1\\\",                        \\\"mlCode\\\" : \\\"sample_ml_code\\\"                    }                }            },            \\\"currentFilterId\\\" : 619,            \\\"currentlyExecutingJobId\\\" : 87,            \\\"invokingAppInstanceId\\\" : 137,            \\\"rootDagContextId\\\" : 2591,            \\\"sessionCode\\\" : \\\"zpss_5_9_307\\\",            \\\"versionTagId\\\" : 488        }    },    \\\"operationsKey\\\" : \\\"200474257\\\",    \\\"tenantDataFolder\\\" : \\\"alpha.tenant.data/qinstance1-tenant1\\\"}"
    os.environ['PYTHON_CONFIG_NODE'] = 'b179a8e34f9f4cb4a319bad9b4e85a5e'
    utils = Utils()
    utils.print("Hello World")
    data = '''
    {
      "name": "John Doe",
      "age": 30,
      "isDeveloper": true,
      "languages": [
        "Python",
        "JavaScript",
        "Java"
      ],
      "address": {
        "street": "123 Main St",
        "city": "Springfield",
        "state": "IL",
        "postalCode": "62701"
      }
    }
    '''
    utils.print(data)



from asw_sdk.GraphQlWebClient import GraphQlWebClient
from asw_sdk.ConfigurationManager import ConfigurationManager
from asw_sdk.RegisterModelProps import RegisterModelProps
import os


class MLModel:
    def __init__(self, override_config=None, overriden_config_node=None):
        self._config_manager = None
        if overriden_config_node and override_config:
            self._config_manager = ConfigurationManager(overriden_config_node, override_config)
        else:
            self._config_manager = ConfigurationManager(os.environ['PYTHON_CONFIG_NODE'])
        self._graphql_web_client = GraphQlWebClient(self._config_manager)
        self._graphql_web_client.refresh_token()

    def register_model(self, props: RegisterModelProps):
        ml_code = self._config_manager.get_ml_code()
        ml_model_api = self._config_manager.get_ml_model_api()
        group_id = self._config_manager.get_current_group_id()
        query = f"""
            query MyQuery {{
              {ml_model_api}(
                filter: {{
                  ml_code: {{ equalTo: "{ml_code}" }}
                  and: {{ group_id: {{ equalTo: "{group_id}" }} }}
                }}
              ) {{
                  nodes {{
                    ml_model_id
                    }}
              }}
            }}
            """
        all_ml_models_for_group_json = self._graphql_web_client.query(query, "query")
        if len(all_ml_models_for_group_json["data"][ml_model_api]["nodes"]) == 0:
            self.create_new_ml_model(props)
        else:
            self.update_ml_model(all_ml_models_for_group_json["data"][ml_model_api]["nodes"][0]["ml_model_id"], props)

    def get_active_model_by_group(self, group_id):
        ml_code = self._config_manager.get_ml_code()
        ml_model_api = self._config_manager.get_ml_model_api()
        query = f"""
            query MyQuery {{
                {ml_model_api}(
                    filter: {{
                        ml_code: {{ equalTo: "{ml_code}" }}
                        and: {{ group_id: {{ equalTo: "{group_id}" }} }}
                    }}
                ) {{
                    nodes {{
                        step_id
                        ml_model_id
                        ml_code
                        job_code
                        is_deleted
                        group_id
                        mlModelVersionsApis(filter: {{ is_active: {{ equalTo: true }} }}) {{
                            nodes {{
                                additional_params_json
                                hyperparameter_json
                                is_active
                                is_deleted
                                ml_model_id
                                ml_model_versions_id
                                model_location
                                run_id
                                run_through
                                target_columns_json
                                training_features_json
                                version
                                mlModelVersionMetricsByMlModelVersionsId {{
                                    nodes {{
                                        chart_type
                                        is_deleted
                                        ml_model_version_metrics_id
                                        ml_model_versions_id
                                        name
                                        value
                                    }}
                                }}
                                description
                            }}
                        }}
                    }}
                }}
            }}
            """

        return self._graphql_web_client.query(query, "query")

    def get_current_active_model(self):
        group_id = self._config_manager.get_current_group_id()
        return self.get_active_model_by_group(group_id)

    def create_new_ml_model(self, props: RegisterModelProps):
        ml_model_api = self._config_manager.get_ml_model_api()
        ml_code = self._config_manager.get_ml_code()
        ml_model_step = self._config_manager.get_currently_executing_step_name()
        step_code = self._config_manager.get_step_code()
        run_through = props.run_through
        run_id = self._config_manager.get_current_run_id()
        training_features = props.training_features
        target_cols = props.target_cols
        group_id = self._config_manager.get_current_group_id()
        additional_params = props.additional_params
        ml_model_version_desc = props.ml_model_version_desc
        hyper_parameters_json = props.hyper_parameters_json
        is_active_string = "true" if props.is_active else "false"
        model_location = props.model_location
        metrics_json = props.metrics_json
        job_code = self._config_manager.get_job_code()
        query = f"""
            mutation MyMutation {{
              create{ml_model_api}(
                input: {{
                  {ml_model_api}: {{
                    group_id: "{group_id}"
                    is_deleted: false
                    ml_code: "{ml_code}"
                    step_id: "{step_code}"
                    job_code: "{job_code}"
                    mlModelVersionsApisUsingMlModelId: {{
                      create: {{
                        additional_params_json: "{additional_params}"
                        description: "{ml_model_version_desc}"
                        hyperparameter_json: "{hyper_parameters_json}"
                        is_active: {is_active_string}
                        is_deleted: false
                        run_through: "{run_through}"
                        model_location: "{model_location}"
                        run_id: "{run_id}"
                        target_columns_json: "{target_cols}"
                        training_features_json: "{training_features}"
                        mlModelVersionMetricsApisUsingMlModelVersionsId: {{
                          create: {metrics_json}
                        }}
                      }}
                    }}
                  }}
                }}
              ) {{
                mlModelApiEdge {{
                  node {{
                    mlModelVersionsApis(last: 1) {{
                      nodes {{
                        ml_model_versions_id
                        ml_model_id
                      }}
                    }}
                  }}
                }}
              }}
            }}
            """
        inserted_ml_model = self._graphql_web_client.query(query, "mutation")
        ml_model_versions_id = \
            inserted_ml_model["data"]["create" + ml_model_api]["mlModelApiEdge"]["node"]["mlModelVersionsApis"][
                "nodes"][0][
                "ml_model_versions_id"]
        ml_model_id = \
            inserted_ml_model["data"]["create" + ml_model_api]["mlModelApiEdge"]["node"]["mlModelVersionsApis"][
                "nodes"][0][
                "ml_model_id"]
        self.update_version(ml_model_id, ml_model_versions_id)

    def update_version(self, ml_model_id, ml_model_versions_id):
        ml_model_versions_api = self._config_manager.get_ml_model_versions_api()
        latest_version_query = f"""
        query MyQuery {{
          {ml_model_versions_api}(filter: {{ ml_model_id: {{ equalTo: "{ml_model_id}" }} }}) {{
            totalCount
          }}
        }}
        """
        latest_version_id = self._graphql_web_client.query(latest_version_query, "query")
        update_version_query = f"""
        mutation MyMutation {{
        update{ml_model_versions_api}(
            input: {{patch: {{version: {latest_version_id["data"][ml_model_versions_api]["totalCount"]} }}, ml_model_versions_id: "{ml_model_versions_id}"}}
        ){{
            clientMutationId
            }}
        }}

        """
        self._graphql_web_client.query(update_version_query, "mutation")

    def update_ml_model(self, ml_model_id, props: RegisterModelProps):
        ml_model_api = self._config_manager.get_ml_model_api()
        run_through = props.run_through
        run_id = self._config_manager.get_current_run_id()
        training_features = props.training_features
        model_location = props.model_location
        target_cols = props.target_cols
        additional_params = props.additional_params
        ml_model_version_desc = props.ml_model_version_desc
        hyper_parameters_json = props.hyper_parameters_json
        is_active_string = "true" if props.is_active else "false"
        metrics_json = props.metrics_json
        query = f"""
        mutation MyMutation {{
          update{ml_model_api}(
            input: {{
              patch: {{
                is_deleted: false
                mlModelVersionsApisUsingMlModelId: {{
                  create: {{
                    additional_params_json: "{additional_params}"
                    description: "{ml_model_version_desc}"
                    hyperparameter_json: "{hyper_parameters_json}"
                    is_active: {is_active_string}
                    is_deleted: false
                    run_through: "{run_through}"
                    model_location: "{model_location}"
                    run_id: "{run_id}"
                    target_columns_json: "{target_cols}"
                    training_features_json: "{training_features}"
                    mlModelVersionMetricsApisUsingMlModelVersionsId: {{
                      create: {metrics_json}
                    }}
                  }}
                }}
              }}
              ml_model_id: "{ml_model_id}"
            }}
          ) {{
            mlModelApiEdge {{
              node {{
                mlModelVersionsApis(last: 1) {{
                  nodes {{
                    ml_model_versions_id
                  }}
                }}
              }}
            }}
          }}
        }}
        """
        inserted_ml_model = self._graphql_web_client.query(query, "mutation")
        ml_model_versions_id = \
            inserted_ml_model["data"]["update" + ml_model_api]["mlModelApiEdge"]["node"]["mlModelVersionsApis"][
                "nodes"][0][
                "ml_model_versions_id"]
        self.update_version(ml_model_id, ml_model_versions_id)


if __name__ == '__main__':
    # ml_model = MLModel(f"""{{
    #                    "TOKEN": "eyJob3N0TmFtZSI6Imh0dHBzOi8vc2Fua2V0MTk5NC5hbHBoYS5sb2NhbDo5OTk4IiwicmVmcmVzaFRva2VuIjoicl9Ncll1dzRZWGo3WnplU3V6Iiwid3NIb3N0TmFtZSI6IndzczovL3NhbmtldDE5OTQud3MuYWxwaGEubG9jYWw6OTk5OC93cyJ9",
    # "tenantDataFolder":"abc",
    #                    "nodes": {{
    #     "65ee6b4abf0743e9988255406c410280": {{
    #         "applicationDataTypeId": 120,
    #         "applicationGroupId": 15,
    #         "cittaAgent": {{
    #             "launcher": {{
    #                 "pipelineConf": {{
    #                     "executionProfileId": "ep1",
    #                     "executionProfileObjectName": "exp_object_name",
    #                     "filterName": "smm-1",
    #                     "groupId": 1,
    #                     "jobCode": "GVGWFHBMKMJUAGR",
    #                     "mlCode": "ml_code1",
    #                     "mlModelApiName": "ml_model_api",
    #                     "mlModelObjectName": "ml_model",
    #                     "mlModelVersionApi": "ml_model_versions_api",
    #                     "mlModelVersionMetricsApi": "ml_model_version_metrics_api",
    #                     "mlModelVersionMetricsObjectName": "ml_model_version_metrics",
    #                     "mlModelVersionObjectName": "ml_model_versions",
    #                     "runId": "test_runId"
    #                 }}
    #             }}
    #         }},
    #         "currentFilterId": 4094,
    #         "currentlyExecutingJobId": null,
    #         "invokingAppInstanceId": 2,
    #         "rootDagContextId": 15,
    #         "sessionCode": "zpsa_1_1_2",
    #         "versionTagId": 15
    #     }}
    # }}
    # }}""", overriden_config_node="65ee6b4abf0743e9988255406c410280")
    ml_model = MLModel()
    print(ml_model.get_current_active_model())
    # props = RegisterModelProps(model_id=None, model_location="//ss1", is_active=True
    #                            , run_through="1"
    #                            , training_features="[{name:'x',value:'1'},{name:'y',value:'2'}]"
    #                            , target_cols="['abc','cde','efg']"
    #                            , additional_params="{}"
    #                            , ml_model_version_desc="version desc 1"
    #                            , hyper_parameters_json="{name:'x',value:'1', age:20, type:'t1'}"
    #                            , metrics_json="[{ chart_type: \"line\", name: \"char_name\", value: \"[{x:100,y:100}]\", is_deleted: false }]")
    #
    # ml_model.register_model(props)



from asw_sdk.WebClient import WebClient


class GraphQlWebClient(WebClient):
    def __init__(self, config_manager):
        super().__init__("datatypes", config_manager, use_app_instance_session_code=True)

    def get_app_graphql_endpoint(self):
        return "/app-api/graphql"

    def query(self, graphql_query, graphql_query_type):
        graphql_query_obj = {
            "query": graphql_query
        }
        app_graphql_endpoint = self.get_app_graphql_endpoint()
        headers = {"graphqlOpType": graphql_query_type}
        return super().post_entity(app_graphql_endpoint, json_data=graphql_query_obj, headers=headers)



class RegisterModelProps:
    def __init__(self, model_id, model_location, is_active, **kwargs):
        self.model_id = model_id
        self.model_location = model_location
        self.is_active = is_active
        for key, value in kwargs.items():
            setattr(self, key, value)



import requests
import backoff
import base64
from http.cookies import SimpleCookie


class WebClient:
    AUTHORIZATION = "Authorization"
    QUERY_PARAM_FOR_APP_INSTANCE_ID = "appInstanceId"
    QUERY_PARAM_BITEMPORAL_PROPS = "biTemporalProps"
    QUERY_PARAM_TIME_LABEL_PROP = "timeLabel"
    QUERY_PARAM_CURRENTLY_EXECUTING_JOB_ID = "currentJobId"
    ML_CODE_PROP = "mlCode"
    GROUP_ID = "groupId"
    AGI_ID = "agiId"

    def __init__(self, base_url, config_manager, use_app_instance_session_code=False):
        self.auth_token = ""
        self.configManager = config_manager
        self._host_name = self.configManager.get_host_name()
        self._session_code = self.configManager.get_app_instance_session_code() if use_app_instance_session_code \
            else self.configManager.get_session_code()
        self._base_url = self._host_name + "/" + base_url + "/" + self._session_code

    def _attach_headers(self, headers):
        if headers is None:
            headers = {}
        # headers[self.AUTHORIZATION] = self.auth_token
        headers["cookie"] = 'auth="'+self.auth_token+'"'
        if self.configManager.get_currently_executing_job_id() is not None:
            headers[self.QUERY_PARAM_CURRENTLY_EXECUTING_JOB_ID] = self.configManager.get_currently_executing_job_id()
        if self.configManager.get_current_app_instance_id() is not None:
            headers[self.QUERY_PARAM_FOR_APP_INSTANCE_ID] = self.configManager.get_current_app_instance_id()

    @backoff.on_exception(backoff.expo, requests.exceptions.RequestException, max_tries=5)
    def get_entity(self, endpoint, params=None, headers=None, set_headers=True):
        if set_headers:
            self._attach_headers(headers=headers)
            url = self._base_url + endpoint
        else:
            url = endpoint
        try:
            response = requests.get(url, params=params, headers=headers, verify=True)
            if response.status_code == 200:
                return response.json()
            else:
                response.raise_for_status()
        except requests.exceptions.RequestException as e:
            raise Exception(f"An HTTP error occurred: {e}")

    @backoff.on_exception(backoff.expo, requests.exceptions.RequestException, max_tries=5)
    def post_entity(self, endpoint, params=None, json_data=None, headers=None):
        self._attach_headers(headers=headers)
        url = self._base_url + endpoint
        try:
            response = requests.post(url, params=params, json=json_data, headers=headers, verify=False)
            if response.status_code == 200:
                return response.json()
            else:
                response.raise_for_status()
        except requests.exceptions.RequestException as e:
            raise Exception(f"An HTTP error occurred: {e}")

    @backoff.on_exception(backoff.expo, requests.exceptions.RequestException, max_tries=5)
    def put_entity(self, endpoint, params=None, json_data=None, headers=None):
        self._attach_headers(headers=headers)
        url = self._base_url + endpoint
        try:
            response = requests.put(url, params=params, json=json_data, headers=headers, verify=False)
            if response.status_code == 200:
                return response.json()
            else:
                response.raise_for_status()
        except requests.exceptions.RequestException as e:
            raise Exception(f"An HTTP error occurred: {e}")

    @backoff.on_exception(backoff.expo, requests.exceptions.RequestException, max_tries=5)
    def delete_entity(self, endpoint, params=None, data=None, headers=None):
        self._attach_headers(headers=headers)
        url = self._base_url + endpoint
        try:
            response = requests.delete(url, params=params, data=data, headers=headers, verify=False)
            if response.status_code == 200:
                return response.json()
            else:
                response.raise_for_status()
        except requests.exceptions.RequestException as e:
            raise Exception(f"An HTTP error occurred: {e}")

    def refresh_token(self):
        headers = {"refresh": self.configManager.get_refresh_token()}
        response = requests.get(self._host_name + "/" + "orgs/auth/refresh", headers=headers, verify=False)
        refresh_cookies = response.headers["set-cookie"]
        cookie = SimpleCookie()
        cookie.load(refresh_cookies)
        authorization_token = cookie.get("auth").value
        self.auth_token = authorization_token



from asw_sdk.ConfigurationManager import ConfigurationManager
from asw_sdk.WebClient import WebClient
from asw_sdk.GraphQlWebClient import GraphQlWebClient
from asw_sdk.MLModel import MLModel
from asw_sdk.Utils import Utils



from distutils.core import setup

setup(
    name='asw_sdk',  # How you named your package folder (MyLib)
    packages=['asw_sdk'],  # Chose the same as "name"
    version='9.0.0',  # Start with a small number and increase it with every change you make
    license='MIT',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='Thotnr ML client to interact with Thotnr Projects',  # Give a short description about your library
    author='Thotnr',  # Type in your name
    author_email='npm-admin@thotnr.com',  # Type in your E-Mail

    download_url='https://github.com/user/reponame/archive/v_01.tar.gz',  # I explain this later on
    keywords=['thotnr', 'ml', 'client'],  # Keywords that define your package best
    install_requires=[  # I get to this in a second
        'backoff',
        'requests',
    ],
    python_requires=">=3.7",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
)



import json
import os
import base64


class ConfigurationManager:
    AUTHORIZATION = "Authorization"
    INVOKING_APP_INSTANCE_ID = "invokingAppInstanceId"
    QUERY_PARAM_CURRENTLY_EXECUTING_JOB_ID = "currentlyExecutingJobId"
    AGI_ID = "applicationGroupId"
    ML_CODE_PROP = "mlCode"
    GROUP_ID = "groupId"
    EP_ID = "executionProfileId"
    EP_OBJECT_NAME = "executionProfileObjectName"
    ML_MODEL_API_NAME = "mlModelApiName"
    ML_MODEL_OBJECT_NAME = "mlModelObjectName"
    CURRENT_RUN_ID = "runId"
    CURRENTLY_EXECUTING_STEP_NAME = "filterName"
    ML_MODEL_VERSIONS_OBJECT_NAME = "mlModelVersionObjectName"
    ML_MODEL_VERSIONS_API = "mlModelVersionApi"
    ML_MODEL_VERSION_METRICS_OBJECT_NAME = "mlModelVersionMetricsObjectName"
    ML_MODEL_VERSION_METRICS_API = "mlModelVersionMetricsApi"
    JOB_CODE = "jobCode"
    SESSION_CODE = "sessionCode"
    TOKEN = "TOKEN"
    HOST_NAME = "hostName"
    PYTHON_CONFIG = "PYTHON_CONFIG"
    AGENT = "cittaAgent"
    LAUNCHER = "launcher"
    PIPELINE_CONF = "pipelineConf"
    TENANT_DATA_FOLDER = "tenantDataFolder"
    REFRESH = "refreshToken"
    OPERATIONS_KEY = "operationsKey"
    FILTER_ID = "currentFilterId"

    def __init__(self, step_code, override_config=None):

        # Load configuration from environment variable and parse JSON
        cleaned_json_string = ""
        if override_config:
            cleaned_json_string = override_config.replace("\\\"", "\"")
        else:
            cleaned_json_string = os.environ[self.PYTHON_CONFIG].replace("\\\"", "\"")
        self._all_node_config = json.loads(cleaned_json_string)
        self._step_code = step_code
        self._json_config = self._all_node_config["nodes"][step_code]

        pipeline_conf = self._json_config.get("%s" % self.AGENT, {}).get("%s" % self.LAUNCHER, {}).get("%s" % self.PIPELINE_CONF, {})
        self._filter_id = self._json_config[self.FILTER_ID] if self.FILTER_ID in self._json_config else None
        self._session_code = self._json_config[self.SESSION_CODE]
        self._agiId = str(self._json_config[self.AGI_ID]) if self.AGI_ID in self._json_config else None

        token_decoded = base64.b64decode(self._all_node_config[self.TOKEN])
        token_obj = json.loads(token_decoded)
        self._token = token_obj[self.REFRESH]
        self._host_name = token_obj[self.HOST_NAME]
        self.tenant_data_folder = self._all_node_config[self.TENANT_DATA_FOLDER]
        self.operations_key = self._all_node_config[self.OPERATIONS_KEY]

        # Extract properties from JSON configuration
        self._app_instance_id = str(self._json_config[self.INVOKING_APP_INSTANCE_ID]) if self.INVOKING_APP_INSTANCE_ID in self._json_config else None
        self._currently_executing_job_id = str(self._json_config[self.QUERY_PARAM_CURRENTLY_EXECUTING_JOB_ID]) if self.QUERY_PARAM_CURRENTLY_EXECUTING_JOB_ID in self._json_config else None

        # Extract properties from pipeline configuration
        self._pipeline_conf_props = self.extract_pipeline_props(pipeline_conf)

    def extract_pipeline_props(self, pipeline_conf):
        properties = {
            "ml_code": pipeline_conf.get(self.ML_CODE_PROP, None),
            "group_id": int(pipeline_conf.get(self.GROUP_ID, None)) if pipeline_conf.get(self.GROUP_ID, None) else None,
            "ep_id": str(pipeline_conf.get(self.EP_ID, None)) if pipeline_conf.get(self.EP_ID, None) else None,
            "ep_object_name": str(pipeline_conf.get(self.EP_OBJECT_NAME, None)) if pipeline_conf.get(self.EP_OBJECT_NAME, None) else None,
            "ml_model_object_name": str(pipeline_conf.get(self.ML_MODEL_OBJECT_NAME, None)) if pipeline_conf.get(self.ML_MODEL_OBJECT_NAME, None) else None,
            "ml_model_api": str(pipeline_conf.get(self.ML_MODEL_API_NAME, None)) if pipeline_conf.get(self.ML_MODEL_API_NAME, None) else None,
            "run_id": str(pipeline_conf.get(self.CURRENT_RUN_ID, None)) if pipeline_conf.get(self.CURRENT_RUN_ID, None) else None,
            "current_step_name": str(pipeline_conf.get(self.CURRENTLY_EXECUTING_STEP_NAME, None)) if pipeline_conf.get(self.CURRENTLY_EXECUTING_STEP_NAME, None) else None,
            "ml_model_versions_object": str(pipeline_conf.get(self.ML_MODEL_VERSIONS_OBJECT_NAME, None)) if pipeline_conf.get(self.ML_MODEL_VERSIONS_OBJECT_NAME, None) else None,
            "ml_model_versions_object_api": str(pipeline_conf.get(self.ML_MODEL_VERSIONS_API, None)) if pipeline_conf.get(self.ML_MODEL_VERSIONS_API, None) else None,
            "ml_model_version_metrics_object_name": str(pipeline_conf.get(self.ML_MODEL_VERSION_METRICS_OBJECT_NAME, None)) if pipeline_conf.get(self.ML_MODEL_VERSION_METRICS_OBJECT_NAME, None) else None,
            "job_code": str(pipeline_conf.get(self.JOB_CODE, None)) if pipeline_conf.get(self.JOB_CODE, None) else None,
            "ml_model_version_metrics_api": str(pipeline_conf.get(self.ML_MODEL_VERSION_METRICS_API, None)) if pipeline_conf.get(self.ML_MODEL_VERSION_METRICS_API, None) else None
        }
        return properties

    def get_tenant_data_folder(self):
        return self.tenant_data_folder

    def get_ml_model_versions_object(self):
        return self._pipeline_conf_props["ml_model_versions_object"]

    def get_ml_model_versions_api(self):
        return self._pipeline_conf_props["ml_model_versions_object_api"]

    def get_currently_executing_step_name(self):
        return self._pipeline_conf_props["current_step_name"]

    def get_ep_object_name(self):
        return self._pipeline_conf_props["ep_object_name"]

    def get_ml_model_object_name(self):
        return self._pipeline_conf_props["ml_model_object_name"]

    def get_ml_model_api(self):
        return self._pipeline_conf_props["ml_model_api"]

    def get_current_run_id(self):
        return self._pipeline_conf_props["run_id"]

    def get_ml_model_version_metrics_object_name(self):
        return self._pipeline_conf_props["ml_model_version_metrics_object_name"]

    def get_ml_model_version_metrics_api(self):
        return self._pipeline_conf_props["ml_model_version_metrics_api"]

    def get_job_code(self):
        return self._pipeline_conf_props["job_code"]

    def get_object_from_app_config(self, key):
        return self._pipeline_conf_props[key]

    def does_app_config_contain_key(self, key):
        return key in self._pipeline_conf_props

    def get_object_from_base_config(self, key):
        return self._json_config[key]

    def does_base_config_contain_key(self, key):
        return key in self._json_config

    def get_current_execution_profile_id(self):
        return self._pipeline_conf_props["ep_id"]

    def get_ml_code(self):
        return self._pipeline_conf_props["ml_code"]

    def get_currently_executing_job_id(self):
        return self._currently_executing_job_id

    def get_app_instance(self):
        return self._app_instance_id

    def get_current_group_id(self):
        return self._pipeline_conf_props["group_id"]

    def get_current_agi_id(self):
        return self._agiId

    def get_host_name(self):
        return self._host_name

    def get_session_code(self):
        return self._session_code

    def get_app_instance_session_code(self):
        parts = self._session_code.split("_")
        parts[0] = "zpsa"
        parts[-1] = self._app_instance_id
        app_instance_session_code = "_".join(parts)
        return app_instance_session_code

    def get_refresh_token(self):
        return self._token

    def get_current_app_instance_id(self):
        return self._app_instance_id

    def get_step_code(self):
        return self._step_code

    def get_operations_key(self):
        return self.operations_key

    def get_filter_id(self):
        return self._filter_id
